define(['require', './currpage', './renderer', './mediator'], function(req, currpage, renderer, mediator){
	"use strict";

	var EVT_QUEUE,
		FILTER_SEPERATOR = '|',
		GLOBAL_EVENT_OBJECTS = {},
		ERROR_NO_FUNCTION = function(id, call, err){
			this.name = 'NoFunctionFoundError';
			this.message = id + ': the function #-' + call + '-# probably fails or is not implemented yet. - ' + err;
		},

		/**
		 * Function <b>inherit</b> will set the prototype of a System object to the given parent object and will instantiate
		 * a new System object. (prototypal Inheritance)
		 *
		 * @param parent Object to set to the prototype of the new System object
		 * @param _consFunc will be a specific constructor function to instantiate manager
		 * @return {System}
		 */
		inherit = function(parent, _consFunc){
			var ins;
			if(_consFunc){
				_consFunc.prototype = parent;
				ins = new _consFunc();
				//you will be able to create an instance from an instance
				_consFunc.prototype.constructor = _consFunc;
			}else{
				var System = function System(){};

				System.prototype = parent;
				ins = new System();
			}
			return ins;
		},

		/**
		 * This function will allow you to extend object through other objects by merging key values into each other.
		 *
		 * @param t The Object TO extend
		 * @param f The Object extend FROM
		 */
		extend = function(t, f, holdOldProps){
			var key;
			if(f){
				if(!holdOldProps){
					for(key in f){
						if(f.hasOwnProperty(key)){
							t[key] = f[key];
						}

					}
				}else{
					for(key in f){
						if(f.hasOwnProperty(key) && !t.hasOwnProperty(key)){
							t[key] = f[key];
						}

					}
				}
			}
		},

		/**
		 * <b>isAvailable</b> searches for a given string within an array or array of objects.
		 *
		 * @param id The string to search for within an array, or array of objects.
		 * @param arr The array or array of objects to search
		 */
		isAvailable = function(id, arr){
			var i, item, ret = false;
			for(i in arr){
				if(arr.hasOwnProperty(i)){
					if(renderer.isArray(arr[i])){
						ret = isAvailable(id, arr[i]);
						if(ret){
							break;
						}
					}else if(arr[i] === id){
						ret = true;
						break;
					}else{
						for(item in arr[i]){
							if(arr[i][item] === id){
								ret = true;
								//isAvailable(id, arr[i]);
								break;
							}
						}
					}
				}
			}
			return ret;
		},

		/**
		 * This function extends the window object with a given globals object.
		 * @param globals The object that should extend the window object.
		 */
		initGlobals = function(globals){
			this.extend(window, globals);
		},

		/**
		 * Function <b>loadLibrary</b> allows you to load a library for specific system.
		 *
		 * @param libraryObjArr Object or Array with the url to
		 */
		loadLibrary = function(libraryObjArr){
			var i, that = this,
				_func = function(library){
					var i, scripts;
					if(true === library.double_check){
						scripts = window.document.querySelectorAll("script");
						for(i = 0; i < scripts.length; i++){
							if(scripts[i].src && -1 !== scripts[i].src.indexOf(library.url)){
								mediator.publish(library.eventName, null, library.channel || that.MEDIATOR_CHANNEL);
								return;
							}
						}
					}
					req([library.url], function(obj){
						mediator.publish(library.eventName, obj || {}, library.channel || that.MEDIATOR_CHANNEL);
					}, function(e){
						mediator.publish("error." + library.eventName, e, library.channel || that.MEDIATOR_CHANNEL);
					});
				};

			if(this.isArray(libraryObjArr)){
				for(i = 0; i < libraryObjArr.length; i += 1){
					_func(libraryObjArr[i]);
				}
			}else if(this.isObject(libraryObjArr)){
				_func(libraryObjArr);
			}
		},

		/**
		 * This function lets you set function- and parameter filters for configuration purposes.
		 * A system will hold system specific- and global filters within the FILTERS object.
		 *
		 * @param filters The object of filters (primarily functions)
		 */
		setFilters = function(filters){

			if(!this.SYSTEM_ID){
				this.GLOBALFILTERS = filters;
				this.FILTERS = filters;
			}else{
				this.extend(filters, this.GLOBALFILTERS);
				this.FILTERS = filters;
			}
		},

		/**
		 * <b>subscribeEvent</b> will let systems subscribe for events triggered by core/mediator.
		 * The function <b>handleEvent</b> will always be executed if one of the events is triggered.
		 *
		 * @param system The system which subscribes the event
		 * @param sysConfig    The configuration given by the site
		 * @param evtName The name of the event which should be subscribed
		 */
		subscribeEvent = function(system, sysConfig, evtName, channel){
			var alreadyFiredEvent, evt, i,
				tmp = sysConfig._eventMap[evtName];

			channel = channel || mediator.MEDIATOR_CHANNEL;

			if(this.isArray(EVT_QUEUE)){
				for(i = 0; i < EVT_QUEUE.length; i += 1){
					alreadyFiredEvent = EVT_QUEUE[i];
					alreadyFiredEvent[2] = alreadyFiredEvent[2] || mediator.DEFAULT_CHANNEL;
					if(this.isObject(tmp) && evtName === alreadyFiredEvent[2]){
						for(evt in tmp){
							if(evt === alreadyFiredEvent[0]){
								handleEvent(alreadyFiredEvent[1], system, sysConfig, evt, evtName, alreadyFiredEvent[0]); //evtName === channel!
								break;
							}
						}
					}else{
						if(evtName === alreadyFiredEvent[0]){
							handleEvent(alreadyFiredEvent[1], system, sysConfig, evtName, alreadyFiredEvent[0]);
							break;
						}
					}
				}
			}

			if(this.isObject(tmp)){
				for(evt in tmp){
					(function(sys, config, evtName, channel){
						mediator.subscribe(evtName, function(eventObj){
							handleEvent(arguments[1], sys, config, evtName, channel);
						}, channel); //evt_name will be the channel to subscribe to
					})(system, sysConfig, evt, evtName);

				}
			}else{
				(function(sys, config, evt, channel){
					mediator.subscribe(evt, function(eventObj){
						handleEvent(arguments[1], sys, config, evt, channel);
					}, channel);
				})(system, sysConfig, evtName, channel);
			}
		},

		/**
		 * <b>subscribeExternalEvents</b> is used if an event or events could be fired by another component
		 * (e.g a videoplayer or a gallery) before the whole system (e.g. including libraries) is loaded
		 * and all internal system events are triggered.
		 *
		 * @param evts Object of events which should be subscribed
		 */
		subscribeExternalEvents = function(evts, channel){
			if(typeof channel == 'object' || typeof channel == 'undefined') channel = null;

			var evt;
			this.extend(this.SYSTEMCONFIG._eventMap, evts);
			for(evt in evts){
				if(evts.hasOwnProperty(evt)){
					this.subscribeEvent(this, this.SYSTEMCONFIG, evt, channel);
				}
			}
		},

		/**
		 * <b>execFilters</b> will execute all filters out of the filterPrefArr through filter functions
		 * integrated within the given system before.
		 *
		 * @param filterPrefArr An array of prefixed filters.
		 * @param sysFilters The available FILTERS(+GLOBALFILTERS) of the system.
		 * @param system A system itself. It will be the 'this' within a filter function.
		 * @param renderObj An object that will be a parameter of the filter.
		 *
		 * @return result The aggregated result of all filters. Can be true or false.
		 */
		execFilters = function(filterPrefArr, sysFilters, system, renderObj){

			var i, j, filterRes, filterRec, filterResRecursive, filterResults = [], filterPrefArrRecursive = [], bracketCount, result, not = false, filter;

			for(i = 0; i < filterPrefArr.length; i += 1){
				filter = filterPrefArr[i];
				//opened filter block
				for(; i < filterPrefArr.length; i += 1){
					filterRec = filterPrefArr[i];
					if(-1 !== filterRec.indexOf("[") || filterRec.indexOf("]") !== -1){

						if(-1 !== filter.indexOf("[")){
							bracketCount = filterRec.split("[").length - 1;

							if(1 < bracketCount){

								var newfilter, newRecursive = [], newResRecursive = [];
								newfilter = filterPrefArr[i];
								newfilter = newfilter.substr(bracketCount - 1);
								i++;
								for(; i < filterPrefArr.length; i += 1){
									//closed filter Block
									if(-1 !== newfilter.indexOf("]")){
										if(bracketCount === newfilter.split("]").length - 1){
											bracketCount = newfilter.split("]").length - 1;
											newfilter = newfilter.slice(0, newfilter.length - bracketCount + 1);
											newRecursive.push(newfilter);
											filterResults.push(execFilters(newRecursive, sysFilters, system, renderObj));
											break;
										}else{
											newRecursive.push(newfilter);
											newfilter = filterPrefArr[i];
										}
									}else{
										newRecursive.push(newfilter);
										newfilter = filterPrefArr[i];
									}
								}
							}else{
								filterRec = filterRec.substr(bracketCount);
								filterPrefArrRecursive.push(filterRec);
								i++;
							}
						}
						for(; i < filterPrefArr.length; i += 1){
							filterRec = filterPrefArr[i];
							//closed filter Block
							if(-1 !== filterRec.indexOf("]")){
								bracketCount = filterRec.split("]").length - 1;
								filterRec = filterRec.slice(0, filterRec.length - bracketCount);
								filterPrefArrRecursive.push(filterRec);
								filterResRecursive = execFilters(filterPrefArrRecursive, sysFilters, system, renderObj);
								break;
							}else if(-1 !== filterRec.indexOf("[")){
								bracketCount = filterRec.split("[").length - 1;
								filterRec = filterRec.substr(bracketCount);
								filterPrefArrRecursive.push(filterRec);
							}else{
								filterPrefArrRecursive.push(filterRec);
							}
						}

						filterResults.push(filterResRecursive);
						filterPrefArrRecursive = [];
					}else{
						filter = filterPrefArr[i];
						break;
					}
				}

				not = ("!" === filter.charAt(0));
				if(not)
					filter = filter.substring(1);

				filterRes = undefined;
				if(filter === ""){
					filterRes = "||"
				}else if("undefined" !== typeof sysFilters[filter] && system.isFunction(sysFilters[filter])){
					filterRes = sysFilters[filter].call(system, renderObj);
				}else if("undefined" !== typeof sysFilters[filter] && typeof sysFilters[filter] === "boolean"){
					filterRes = sysFilters[filter];
				}else if(typeof filter === "boolean"){
					filterRes = filter;
				}else{
					break;
				}

				if(not)
					filterRes = !filterRes;

				if("undefined" !== typeof filterRes){
					filterResults.push(filterRes);
				}
			}

			return evaluateFilterResults(filterResults, system);
		},

		evaluateFilterResults = function(filterResults, system){
			var i, result;
			for(i = 0; i < filterResults.length; i += 1){
				if(system.isArray(filterResults[i])){
					filterResults[i] = evaluateFilterResults(filterResults[i], system);
				}
				//OR or AND
				if('||' === filterResults[i]){
					i += 1;
					result = result || filterResults[i];
				}else{
					result = typeof result === "undefined" ? filterResults[i] : result && filterResults[i];
				}

			}
			return result;
		},

		/**
		 * This function will analyse the parameters given to the 'configured function'.
		 * If there are filter prefixes within object keys they will be executed and parameters will be
		 * edited by deletion or renaming before they are given to the  'configured function'.
		 *
		 * @param system A system object
		 * @param params The params which have to be filtered perhaps.
		 * @param renderObj An object that will be a parameter of the filter.
		 */
		execSubParamFilters = function(system, params, renderObj){
			var i, iSplLength, iSplit, filtersResult, filters, wasFilterd;
			if(system.isObject(params)){
				for(i in params){
					if(params.hasOwnProperty(i)){
						iSplit = i.split('|');
						filters = [];
						wasFilterd = false;
						filtersResult = true;

						if(iSplit.length > 1){
							extractFilters(iSplit, filters);
							filtersResult = execFilters(filters, system.FILTERS, system, renderObj);
							wasFilterd = true;
						}
						if(!filtersResult){
							delete (params[i]);
						}else{
							if(wasFilterd){
								iSplLength = iSplit.length - 1;
								params[iSplit[iSplLength]] = params[i];
								delete (params[i]);
								execSubParamFilters(system, params[iSplit[iSplLength]], renderObj);
							}else{
								execSubParamFilters(system, params[i], renderObj);
							}
						}
					}
				}
			}
			return params;
		},

		/**
		 * <b>extractFilters</b> analyses a split (by FILTER_SEPERATOR) string (an Array) for filters
		 * and push their ids to an array. The last entry of the split array is the function or parameter name itself.
		 *
		 * @param strSplit The string split by FILTER_SEPERATOR
		 * @param filters An array to be filled with filter ids
		 *
		 * @return The final function or parameter name
		 */
		extractFilters = function(strSplit, filters, isRendered){
			var strSplitLength, i, specialFilter = '!render';

			strSplitLength = strSplit.length;
			if(strSplitLength > 1){
				for(i = 0; i < strSplitLength - 1; i += 1){//last itme will be the function call itself
					if(specialFilter === strSplit[i]){
						isRendered = false;
					}else{
						filters.push(strSplit[i]);
					}
				}
			}
			return strSplit[strSplitLength - 1];
		},

		/**
		 * <b>execEventHandler</b>
		 *
		 * @param evtHandler
		 * @param system
		 * @param sysConfig
		 * @param renderObj
		 */
		execEventHandler = function(evtHandler, system, sysConfig, renderObj){
			var i, funcCall, funcCallSplit, funcParams, funcParam, funcParamSplit, func_filters, param_filters, isParamToBeRendered, funcParamsObjects, paramFiltersResult = true, filtersResult = true, tmp = evtHandler.split('(');

			funcCall = tmp[0];
			funcCallSplit = funcCall.split(FILTER_SEPERATOR);
			funcParams = tmp[1] ? tmp[1].slice(0, -1) : '';
			funcParams = funcParams.replace(/\s/g, '').split(',');
			func_filters = [];
			param_filters = [];

			if(system.EVENT_OBJECTS){
				extend(renderObj, system.EVENT_OBJECTS);
			}else{
				extend(renderObj, GLOBAL_EVENT_OBJECTS);
			}

			if(funcCallSplit.length > 1){
				funcCall = extractFilters(funcCallSplit, func_filters);
			}

			if(func_filters.length > 0){//if there are filters to this function execute them
				filtersResult = execFilters(func_filters, system.FILTERS, system, renderObj);
			}

			if(system.isArray(funcParams)){
				if(true === filtersResult){//default is true, if on filter returns false it will be false;
					funcParamsObjects = [];
					for(i = 0; i < funcParams.length; i += 1){
						isParamToBeRendered = true;
						funcParam = funcParams[i];
						param_filters = [];
						funcParamSplit = funcParam.split(FILTER_SEPERATOR);

						if(funcParamSplit.length > 1){
							isParamToBeRendered = extractFilters(funcParamSplit, param_filters);
							funcParam = funcParamSplit[funcParamSplit.length - 1];
						}

						if(param_filters.length > 0){
							paramFiltersResult = execFilters(param_filters, system.FILTERS, system, renderObj);
						}

						if(true === paramFiltersResult){
							funcParam = sysConfig.params[funcParam];

							if(funcParam){
								funcParam = execSubParamFilters(system, funcParam, renderObj);
								if(true === isParamToBeRendered){
									funcParam = system.render(funcParam, renderObj);
								}
								funcParamsObjects.push(funcParam);
							} else {
								funcParamsObjects.push(funcParam);
							}
						}
					}

					try{
						//fill the rest with undefined
						for(i=funcParamsObjects.length; i < system[funcCall].length; i++){
							funcParamsObjects.push(undefined);
						}
						funcParamsObjects.push(renderObj);
						system[funcCall].apply(system, funcParamsObjects);
					}catch(e){
						throw new ERROR_NO_FUNCTION(system.SYSTEM_ID, funcCall, e);
					}
				}
			}
		},

		/**
		 * <b>handleEvent</b>
		 *
		 * @param eventObj
		 * @param system
		 * @param sysConfig
		 * @param evtName
		 */
		handleEvent = function(eventObj, system, sysConfig, evtName, channel){
			var i, renderObj = {
				currpage: system.getCurrpage(),
				eventObj: eventObj,
				params: sysConfig.params
			}, evtHandler;

			for(i = 0; i < EVT_QUEUE.length; i += 1){
				var alreadyFiredEvent = EVT_QUEUE[i];
				alreadyFiredEvent[2] = alreadyFiredEvent[2] || mediator.DEFAULT_CHANNEL;

				if(evtName === alreadyFiredEvent[0] && alreadyFiredEvent[3] == channel){
					//return;
				}

				if(evtName === alreadyFiredEvent[0]){
					alreadyFiredEvent[3] = channel;
				}
			}

			if(channel){
				evtHandler = sysConfig._eventMap[channel][evtName];
			}else{
				evtHandler = sysConfig._eventMap[evtName];
			}

			if(!evtHandler) evtHandler = sysConfig._eventMap[evtName];

			if(system.isArray(evtHandler)){
				for(i = 0; i < evtHandler.length; i += 1){
					execEventHandler(evtHandler[i], system, sysConfig, renderObj);
				}
			}else if(system.isString(evtHandler)){
				execEventHandler(evtHandler, system, sysConfig, renderObj);
			}else if(system.isFunction(evtHandler)){
				evtHandler.apply(system, [renderObj, evtName]);
			}
		},

		/**
		 * References the external evt_queue to EVT_QUEUE for internal use.
		 *
		 * @param evt_queue Array of events triggered before
		 */
		setEventQueue = function(evt_queue){
			EVT_QUEUE = evt_queue || [];
		},

		/**
		 * <b>setSystemConfig</b> sets the Configuration of the system into the system itself.
		 *
		 * @param sysCfg The system specific Configuration part of the main Configuration
		 */
		setSystemConfig = function(sysCfg){
			this.SYSTEMCONFIG = sysCfg;
		},


		readCookie = function(cookie){
			var content = '',
				name = '',
				start,
				end;
			if(document.cookie.length > 0){
				name = cookie + '=';
				start = document.cookie.indexOf(name);
				end = 0;
				if(start > -1){
					start += name.length;
					end = document.cookie.indexOf(";", start);
					if(end < start){
						end = document.cookie.length;
					}
					content = document.cookie.substring(start, end);
				}
			}
			return unescape(content);
		},


		writeCookie = function(name, value, expires, path){
			expires = null;

			if(typeof(path) === 'undefined'){
				path = '/';
			}

			if(typeof(encodeURIComponent) == 'function'){
				value = encodeURIComponent(value);
			}else{
				value = escape(value);
			}
			var stringToSet = name + '=' + value;

			if(path !== ''){
				stringToSet += ';path=' + path;
			}
			document.cookie = stringToSet;
		},

		/**
		 * <b>setEventObjects</b> will add special self defined objects to the possible render context of variables
		 * given to handler functions. There are possiblities to give objects, strings or arrays within evt_objects.
		 * Strings will be interpreted as a module to required. Objects will be taken as they are. Arrays will be
		 * interpreted as a row of ['moduleToBeRequired', 'functionToBeCalled', 'parameterGivenToFunction']
		 *
		 *
		 * @param evt_objects The objects added to the render context.
		 */
		setEventObjects = function(evt_objects){
			var i;

			if(this.SYSTEM_ID === null){
				if(this.isObject(evt_objects)){
					for(i in evt_objects){
						if(this.isString(evt_objects[i])){
							(function(i){
								req([evt_objects[i]], function(obj){
									GLOBAL_EVENT_OBJECTS[i] = obj;
								});
							})(i);
						}else if(this.isArray(evt_objects[i])){
							(function(i, that){
								req([evt_objects[i][0]], function(obj){
									var appl = [];
									appl.push(evt_objects[i][2] ? evt_objects[i][2] : "");
									GLOBAL_EVENT_OBJECTS[i] = obj[evt_objects[i][1]].apply(null, appl);
								});
							})(i, this);
						}else{
							GLOBAL_EVENT_OBJECTS[i] = evt_objects[i];
						}
					}
				}
			}else{
				if(this.isObject(evt_objects)){
					this.EVENT_OBJECTS = {};
					for(i in evt_objects){
						if(this.isString(evt_objects[i])){
							(function(i, that){
								req([evt_objects[i]], function(obj){
									that.EVENT_OBJECTS[i] = obj;
									extend(that.EVENT_OBJECTS, GLOBAL_EVENT_OBJECTS);
								});
							})(i, this);
						}else if(this.isArray(evt_objects[i])){
							(function(i, that){
								req([evt_objects[i][0]], function(obj){
									var appl = [];
									appl.push(evt_objects[i][2] ? evt_objects[i][2] : "");
									that.EVENT_OBJECTS[i] = obj[evt_objects[i][1]].apply(null, appl);
									extend(that.EVENT_OBJECTS, GLOBAL_EVENT_OBJECTS);
								});
							})(i, this);
						}else{
							this.EVENT_OBJECTS[i] = evt_objects[i];
							extend(this.EVENT_OBJECTS, GLOBAL_EVENT_OBJECTS);
						}
					}
				}
			}
		},

		getQueryParam = function (paramName, queryParamString) {
			queryParamString = queryParamString.split("+").join(" ");
			var param = {}, tokens,
				exp = /[?&;](.+?)=([^&;]*)/g;
			while (tokens = exp.exec(queryParamString)) {
				param[decodeURIComponent(tokens[1])] = decodeURIComponent(tokens[2]);
			}
			if(paramName) {
				param = param[paramName] || "";
			}
			return param;
		},

		getElementsByClassName = function(node, className){
			var elements = [],
				i, j, re, els;
			if(node.getElementsByClassName && "function" === typeof node.getElementsByClassName){
				elements = node.getElementsByClassName(className);
			} else if(node.querySelectorAll){
				elements = node.querySelectorAll("."+className);
			} else {
				re = new RegExp('(^| )'+className+'( |$)' );
				els = node.getElementsByTagName("*");
				for(i=0,j=els.length; i<j; i++){
					if(re.test(els[i].className)){
						elements.push(els[i]);
					}
				}
			}
			return elements;
		},

		getCharacterSet = function(){
			var charSet = '';
			if(document.charset){
				charSet = document.charset;
			} else if(document.characterSet){
				charSet = document.characterSet;
			}
			return charSet.toUpperCase();
		},

		exec = function(){

		};

	ERROR_NO_FUNCTION.prototype = Error.prototype;

	return {
		SYSTEM_ID: null,
		setEventObjects: setEventObjects,
		inherit: inherit,
		isArray: renderer.isArray,
		isFunction: renderer.isFunction,
		isString: renderer.isString,
		isObject: renderer.isObject,
		isAvailable: isAvailable,
		extend: extend,
		readCookie: readCookie,
		getQueryParam: getQueryParam,
		writeCookie: writeCookie,
		render: renderer.render,
		loadLibrary: loadLibrary,
		setSystemConfig: setSystemConfig,
		subscribeEvent: subscribeEvent,
		subscribeExternalEvents: subscribeExternalEvents,
		handleEvent: handleEvent,
		getCharacterSet: getCharacterSet,
		getCurrpage: currpage.get,
		initGlobals: initGlobals,
		setFilters: setFilters,
		setEventQueue: setEventQueue,
		getElementsByClassName: getElementsByClassName,
		exec: exec
	};
});
