(function () {

  if ( typeof window.CustomEvent === "function" ) return false;

  function CustomEvent ( event, params ) {
    params = params || { bubbles: false, cancelable: false, detail: undefined };
    var evt = document.createEvent( 'CustomEvent' );
    evt.initCustomEvent( event, params.bubbles, params.cancelable, params.detail );
    return evt;
   }

  CustomEvent.prototype = window.Event.prototype;

  window.CustomEvent = CustomEvent;
})();
/**
 * Mediator: Interface module to let other Modules
 * communicate via publish/subscribe functions. (Wrapped in UMD-Pattern)
 *
 * @version 2.0
 */
(function(root, factory){
	if(typeof define === "function" && define.amd){
		define(['module', 'component/postal/postal', 'underscore'], factory);
	}
}(this, function(module, postal, _){

	var channels = {},
		subscribers = {},
		evt_queue = [],
		DEFAULT_CHANNEL = "MEDIATOR", //this will be the default channel where all events are running over if no channel is set

		init = function(){
			if(window._mediatorChannels){
				setChannels(window._mediatorChannels);
			}

			if(window._mediatorEvt_Quene){
				setEventQuene(window._mediatorEvt_Quene)
			}

			createChannel(DEFAULT_CHANNEL);

			window._mediatorChannels = getChannels();
			window._mediatorEvt_Quene = getEventQueue();
		},

		preparePreviousChannelEvents = function(channel){
			var i, tmp;
			for(i in subscribers){
				tmp = subscribers[i];
				if(tmp.toAllChannels){
					subscribe(tmp.topic, tmp.callback, channel);
				}
			}
		},
		/**
		 *    Function to create an own channel within postal.js library
		 *
		 *  @param {string|string[]} channel - channel(s) to be created
		 */
		createChannel = function(channel){
			var i;
			if(_.isString(channel)){
				channels[channel] = postal.channel(channel);
				subscribe(function(){
					evt_queue.push([arguments[1].topic, arguments[1], arguments[1].channel]);
				}, channel);
				preparePreviousChannelEvents(channel);
				return channels[channel];
			}else if(_.isArray(channel)){
				for(i = 0; i < channel.length; i++){
					channels[channel[i]] = postal.channel(channel[i]);
					subscribe(function(){
						evt_queue.push([arguments[1].topic, arguments[1], arguments[1].channel]);
					}, channel[i]);
					preparePreviousChannelEvents(channel[i]);
				}
			}

		},

		/**
		 *    Publish to a specific channel a specific event
		 *
		 * @param {string} event - event to be published
		 * @param {*} data - data to be given to the callback function of the event
		 * @param {string|string[]} channel - channel(s) to publish to
		 */
		publish = function(event, data, channel){ //channel optional
			var i, ch;

			channel = channel || DEFAULT_CHANNEL;

			var singleCustomEvent = new CustomEvent('mediator', {
			    detail: {
			        data: data,
			        event: event,
			        channel: channel
			    }
			});
			window.dispatchEvent(singleCustomEvent);

			var customEvent = new CustomEvent('mediator.' + event, {
                detail: {
					data: data,
					channel: channel
				}
            });

            window.customMediatorEvents = window.customMediatorEvents || [];
            window.customMediatorEvents.push(event);
            window.dispatchEvent(customEvent);

			if(_.isString(channel)){
				if(!channels[channel]) createChannel(channel);
				channels[channel].publish(event, data);
			}else if(_.isArray(channel)){//multiple channels
				for(i = 0; i < channel.length; i++){
					ch = channel[i];
					if(!channels[ch]) createChannel(ch);
					channels[ch].publish(event, data);
				}
			}
		},

		publishOnce = function(event, data, channel){
			var i=0;
			if(_.isArray(channel)){
				for(i = 0; i < channel.length; i++){
					if(getTriggerCount(event, channel[i]) === 0){
						publish(event, data, channel[i]);
					}
				}
			} else {
				if(getTriggerCount(event, channel) === 0){
					publish(event, data, channel);
				}
			}
		},

		publishCrossContext = function(event, data, channel, queue){
			queue.push([module.id, "publish", event, data, channel]);
		},

		/**
		 * Subscribe to a specific event on a channel
		 *
		 * @param {string} event - name of the event to be subscribed
		 * @param {function} func - callback function if the event is triggered
		 * @param {string|string[]} channel - channel or channels to subscribe your event to
		 */
		subscribe = function(event, func, channel){ //channel optional
			var i, ch, sub,
				toAllChannels = false;
			channel = channel || DEFAULT_CHANNEL;
			event = event === '*' ? '#' : event;
			if(_.isFunction(event)){
				if(_.isString(func) || _.isArray(func)) channel = func;
				func = event;
				event = "#";
			}

			if('#' === channel || '*' === channel){
				toAllChannels = true;
				channel = getAvailableChannels();
			}

			if(_.isString(channel)){ //one channel
				if(!channels[channel]) createChannel(channel);
				sub = channels[channel].subscribe(event, func);
				subscribers[channel + '|' + event] = sub;
			}else if(_.isArray(channel)){ //multiple channels
				for(i = 0; i < channel.length; i++){
					ch = channel[i];
					if(!channels[ch]) createChannel(ch);
					sub = channels[ch].subscribe(event, func);
					sub.toAllChannels = toAllChannels;
					subscribers[ch + '|' + event] = sub;
				}
			}
		},

		/**
		 * The function will let you unsubscribe from a specific event on a specific channel
		 *
		 * @param {string} event - name of the event to be unsubscribed
		 * @param {string|string[]} channel - channel or channels to unsubscribe your event from
		 */
		unsubscribe = function(event, channel){
			channel = channel || DEFAULT_CHANNEL;
			if(!channels[channel]){
				createChannel(channel);
			}
			if(subscribers[channel + '|' + event]){
				subscribers[channel + '|' + event].unsubscribe();
			}
		},

		/**
		 * This function will provide functionality to get all channels available
		 *
		 * @return {array} Array of strings contains the amount of channels available
		 */
		getAvailableChannels = function(){
			var i, channelArr = [];
			for(i in channels){
				channelArr.push(channels[i].channel);
			}
			return channelArr;
		},

		getTriggerCount = function(evt_name, channel, cb) {
			var i, count = 0, tmp;
			channel = channel || DEFAULT_CHANNEL;
			for (i = 0; i < evt_queue.length; i++) {
				tmp = evt_queue[i];
				if (evt_name === tmp[0] && (channel === tmp[2] || "*" === channel || "#" === channel)) {
					count++;
				}
			}

			if(cb) return cb(count);
			return count;
		},

		getEventQueue = function(){
			return evt_queue;
		},

		setEventQuene = function(quene){
			evt_queue = quene;
		},

		getChannels = function(){
			return channels;
		},

		setChannels = function(chs){
			channels = chs;
		},

		getChannel = function(name){
			if(!channels[name]) createChannel(name);
			return channels[name];
		},

		setChannel = function(name, ch){
			channels[name] = ch;
		};

	//initalize
	init();

	return {
		publish: publish,
		publishOnce: publishOnce,
		publishCrossContext: publishCrossContext,
		subscribe: subscribe,
		unsubscribe: unsubscribe,
		createChannel: createChannel,
		getEventQueue: getEventQueue,
		setEventQuene: setEventQuene,
		getChannel: getChannel,
		setChannel: setChannel,
		getChannels: getChannels,
		setChannels: setChannels,
		getTriggerCount: getTriggerCount,
		EVT_QUEUE: evt_queue,
		DEFAULT_CHANNEL: DEFAULT_CHANNEL,
		getAvailableChannels: getAvailableChannels
	};
}));
