// The component loads outbrain recommendations, renders the returned json via **underscore**
// and appends it to the element. It also takes care of replacing the original url with
// the outbrain redirect url, when the user clicks on a rendered recommendation link.
//
// ##### Options *(object)*:
// - **key:** "myKey"  
// *Used for namespacing the events*
// - **templateId:** "#tpl-outbrain"  
// *Id of the underscore template inside the dom* 
// - **scrollThreshold:** 400  
// *Scrolling distance when the widget gets loaded*
// - **title:** "Widget Title"  
// *The title which gets displayed above the recommendations*
// - **permalink:** window.location.href  
// *The url which gets submitted to outbrain in order to get recommendations*
//
// ##### Example 1: 
// ###### Default implementation
// ```
// $options = array("widgetId" => "AR_11");
// <div class="OutbrainWidget" 
//   data-outbrain='<?= json_encode($options); ?>'>
// </div>
// ```
// ##### Example 2:
// ###### Widget with custom text and scroll threshold
// ```
// $options = array(
//  "widgetId" => "AR_12", 
//  "title" => "Interessantes aus dem Web",
//  "scrollThreshold" => 200
// );
// <div class="OutbrainWidget" 
//   data-outbrain='<?= json_encode($options); ?>'>
// </div>
// ```
//
define( [ 'jquery', 'underscore', 'component/core/mediator', 'viewportService', 'http://widgets.outbrain.com/outbrain.js' ],
  function( $, _, mediator, viewportService ) {

    'use strict';

    var defaults = {
      templateId: '#tpl-outbrain',
      scrollThreshold: 1500,
      title: 'Was sie noch interessieren könnte',
      permalink: 'http://www.chip.de/news/Normale-Panorama-News-Speaking-URL_83725311.html', //window.location.href,
      widgetId: ''
    };

    function Outbrain( selector ) {
      this.$selector = $( selector );

      if ( !( this instanceof Outbrain ) ) {
        return new Outbrain();
      }

      if ( !this.$selector.data( 'Outbrain' ) ) {
        this.init();
      }
    }

    // #### init()
    //
    // Makes sure the element won't get initialized twice by saving the component into 
    // `element.data( 'Outbrain' )`.  
    // Fires `bindEvents()` and sets up subscription for the **viewportService** in order
    // to load the widget on demand.
    //
    Outbrain.prototype.init = function() {
      this.$selector.data( 'Outbrain', this );
      this.params = this.$selector.data( 'outbrain' );
      this.params = _( this.params ).isObject() ? this.params : {};
      this.settings = _( this.params).defaults( defaults );
      this.key = this.settings.key || _( 'Outbrain_' ).uniqueId();

      this.template = _( $( this.settings.templateId ).html() ).template();
      
      this.$widget = this.$selector;

      this.bindEvents();
      this.subscribeToViewportService();
    };

    // #### bindEvents()
    //
    // Binds a namespaced version of `touchstart` and `click` events
    // to the links inside of the rendered widget to redirect the urls
    // over outbrains redirect service.
    //
    Outbrain.prototype.bindEvents = function() {
      this.$widget
        .on( 'touchstart.' + this.key, 'a', $.proxy( this.redirectUrl, this ) )
        .on( 'click.' + this.key, 'a', $.proxy( this.redirectUrl, this ) );
    };

    // #### redirectUrl()
    //
    // Unbinds the event listeners for `touchstart` and `click`
    // replaces the url with the redirect url and triggers a 
    // normal `click` to load the link
    //
    Outbrain.prototype.redirectUrl = function() {
    	var $link = $( event.target ).closest( 'a' );

      event.preventDefault();
      event.stopPropagation();

      $link
        .off( 'touchstart.' + this.key )
        .off( 'click.' + this.key )
        .attr( 'href', $link.data( 'outbrain-redirect' ) )
        .trigger( 'click' );
    };

    // #### subscribeToViewportService()
    //
    // Subscribes to **viewportService** and listens to any changes of 
    // the current viewport and triggers `loadRecommendations()`.
    //
    Outbrain.prototype.subscribeToViewportService = function() {
      mediator.subscribe( 'change' , $.proxy( this.loadRecommendations, this ), 'viewportService' );
    };

    // #### loadRecommendations()
    //
    // Loads outbrain recommendations if the user scrolls towards
    // the widget. Before loading the recommendations a helper class
    // is added to prevent another call to outbrain.
    //
    Outbrain.prototype.loadRecommendations = function() {
      var viewport = viewportService.getViewport(),
        widgetOffsetTop = this.$widget.offset().top,
        loadWidget = viewport.scrollOffsetTop > ( widgetOffsetTop - this.settings.scrollThreshold );

      if ( !loadWidget || this.$widget.hasClass( 'is-loaded' ) ) {
        return;
      }

      this.$widget
        .addClass( 'is-loaded' );

      window.OBR.extern.callRecs( {
        'permalink': this.settings.permalink,
        'widgetId': this.settings.widgetId
      }, $.proxy( this.renderWidget, this ) );
    }

    // #### renderWidget()
    //
    // Renders the widget as a callback function for `loadRecommendations()`.
    // For rendering *underscores* template system is used and afterwards 
    // appended to the element.
    //
    Outbrain.prototype.renderWidget = function( result ) {
      var html = this.template( {
        title: this.settings.title,
        items: result.doc
      } );

      this.$widget
        .append( html );

      mediator.publish( 'ui.init' );
    };

    // Initialization happens when **mediator** publishes the event `ui.init`.
    mediator.subscribe( 'ui.init', function() {
      $( '[data-outbrain]' ).each( function() {
        new Outbrain( this );
      } );
    } );

    return Outbrain;
  } );