define( [ 'jquery', 'underscore', 'viewportService', 'component/core/mediator', 'raven', 'LazyImage', 'hammerjs', 'hammerjs-jquery' ],
    function( $, _, viewportService, mediator, Raven, LazyImage, Hammer ) {

    'use strict';

    var defaults = {
        selCarousel: 'WidgetBC-Carousel',
        selList: 'WidgetBC-Carousel__List',
        selArrow: 'WidgetBC-Carousel__Arrow',
        selArrowPrev: 'WidgetBC-Carousel__Arrow--Prev',
        selArrowNext: 'WidgetBC-Carousel__Arrow--Next',
        selDisabled: 'is-disabled',
        selSlide: 'WidgetBC-Carousel__Slide',
        spacing: 5,
        timeout: 200
    }, component = {
        name: 'WidgetBc',
        slug: 'widget-bc',
        channel: 'ui.widget-bc'
    };

    function WidgetBc( selector ) {
        this.$selector = $( selector );

        if ( !( this instanceof WidgetBc ) ) {
            return new WidgetBc();
        }

        if ( !this.$selector.data( component.name ) ) {
            try {
                this.init();
            } catch( err ) {
                Raven.captureException( err, {
                    extra: {
                        module: component.name
                    }
                });
            }
        }
    }

    // #### init() 
    //
    // Makes sure the element won’t get initialized twice by saving the component 
    // into element.data( 'LazyImage' ).  
    // Triggers `chooseLayout()` to setup the right layout depending on the current viewport.
    // Binds *touchstart*, *click* and *swipe* to the list, the prevNext buttons and the navigation
    // buttons. In the end it subscribes to **mediator* to sync with other **WidgetBc** and with
    // the **viewportService** to trigger `chooseLayout()` again.
    // 
    WidgetBc.prototype.init = function() {

        var args = this.$selector.data( component.slug ),
            params = _( args ).isObject() ? args : {};

        this.settings   = _( params ).defaults( defaults );
        this.key        = this.settings.key || _( component.slug + '_' ).uniqueId();

        this.slide = 0;
        this.position = 0;
        this.delta = this.settings.timeout;
        this.$WidgetBc = this.$selector;
        this.$list = this.$WidgetBc.find( '.'+this.settings.selList );
        this.$slides = this.$WidgetBc.find( '.'+this.settings.selSlide );
        this.$prevNext = this.$WidgetBc.find( '.'+this.settings.selArrow );
        this.$next = this.$WidgetBc.find( '.'+this.settings.selArrowNext );
        this.$prev = this.$WidgetBc.find( '.'+this.settings.selArrowPrev );
        this.$stage = this.$WidgetBc.find( '.'+'WidgetBC-Carousel__Stage' );

        this.chooseLayout();
        this.bindEvents();
        this.subscribeToMediator(); 

        this.$selector.data( component.name, this );
    };


    // #### chooseLayout()
    //
    // Chooses the right layout. Checks in which size *(small, medium, large)* the page currently is
    // displayed by including the **viewportService** and then fetches the corresponding
    // part of `this.settings[ view ]`. After calculating the widths of slides and list it
    // generates the total pages and then triggers `renderPagination()` 
    // to display counter and pagination and finally triggers `goTo()` to jump to the very 
    // first slide.
    //
    WidgetBc.prototype.chooseLayout = function() {

        var view = viewportService.getViewport().viewportRange;

        this.view = view;
        this.width = this.$WidgetBc.outerWidth();
        this.slides = this.$slides.length;
        this.slide = ( this.slide > this.slides ) ? this.slides : this.slide;
        this.slidesWidth = this.$slides.eq(0).outerWidth();
        this.slidesPerPage = Math.floor(this.width / (this.slidesWidth));
        
        this.offsetWidth = this.slidesWidth + this.settings.spacing;
        this.listWidth = (this.slides * this.offsetWidth) - this.settings.spacing;

        this.$WidgetBc
        .addClass( 'is-loaded' );

        this.$list
        .width( this.listWidth );

        this.goTo( this.slide );
    };

    // #### goTo()
    //
    // Takes care of loading the correct slide and calculating the position
    // of the list, which gets translated via css3 to the left and right. 
    // If used as navigation, the currently active slide gets a css helper
    // class to make it visible. The styling itself is not part of this
    // component and has to be done seperately. If syncing the WidgetBc
    // with another is enabled it invokes publishing a **mediator** event of the keys to sync 
    // and the current slide which will trigger a `goTo()` for WidgetBcs to be synced.
    //  
    WidgetBc.prototype.goTo = function ( slide ) {
        var lastSlide = this.slides - this.slidesPerPage,
            disabledSelector = this.settings.selDisabled;

        if ( slide >= lastSlide ) {
            this.slide = lastSlide;
        } else if ( slide <= 0 ) {
            this.slide = 0;
        } else {
            this.slide = slide;
        }


        this.$prev
            .toggleClass( disabledSelector, this.slide < 1 );

        this.$next
            .toggleClass( disabledSelector, this.slide >= lastSlide );


            if (this.slide >= lastSlide) {
                // if there are less items than the thing is wide
                this.position = Math.max(this.listWidth - this.width, 0);
            } else {
                this.position = (this.offsetWidth) * this.slide;
            }

            this.$list
                .css( 'transform', 'translate(' + (this.position * -1) + 'px, 0px)' );
    };

    // #### bindEvents()
    //
    // Binds a namespaced version of `touchstart` and `click` events
    // to the next and previous buttons and the navigation links *(readded if the viewportRange changes)*.
    // If the WidgetBc serves as navigation (i.e. thumbnails) for a another WidgetBc, the
    // slides itself get a click and touch handler applied to activate the current slide. If
    // it does not serve as navigation, the list gets `pan` events bound via **hammerjs**.
    // to enable swiping through the elements.
    //
    WidgetBc.prototype.bindEvents = function() {
        this.$prevNext
            .on( 'touchstart.' + this.key, $.proxy( this.skipToSlide, this ) )
            .on( 'click.' + this.key, $.proxy( this.skipToSlide, this ) );

        if ( this.isNavigation ) {
            this.$slides
                .on( 'touchstart.' + this.key, $.proxy( this.activateSlide, this ) )
                .on( 'click.' + this.key, $.proxy( this.activateSlide, this ) );
        } else {
            this.$WidgetBc.hammer()
                .on( 'panright panleft', $.proxy( this.stickToFinger, this ) )
                .on( 'panend', $.proxy( this.fixPosition, this ) );
        }
    };

    // #### skipToSlide()
    // 
    WidgetBc.prototype.skipToSlide = function( event ) {

        var $button = $( event.target ).closest( '.WidgetBC-Carousel__Arrow' );

        if ( $button.hasClass( 'is-disabled') ) {
            return;
        } else if ( $button.hasClass( 'WidgetBC-Carousel__Arrow--Next' ) ) {
            this.goTo( this.slide + 1 );
        } else if ( $button.hasClass( 'WidgetBC-Carousel__Arrow--Prev' ) ) {
            this.goTo( this.slide - 1 );
        }

        event.stopPropagation();
        event.preventDefault();
    };



    WidgetBc.prototype.reset = function( event ) {
        this.page = 0;
        this.slide = 0;
        this.position = 0;

        this.goTo( this.slide );
    };

    WidgetBc.prototype.stickToFinger = function( event ) {

        if ( event.gesture.pointerType === 'mouse' ) {
            return;
        }

        this.$list
            .css( 'transform', 'translate3d(' + (  this.position + event.gesture.deltaX * 1.5 ) +'px, 0px, 0px)');
    };

    WidgetBc.prototype.synchronize = function( slide ) {
        if ( this.slide !== slide ) {
            this.goTo( slide );
        }
    };

    // #### subscribeToViewportService()
    //
    // Subscribes to changes of the viewport and triggers 'loadImage()'
    WidgetBc.prototype.subscribeToMediator = function() {
        mediator.subscribe( this.key, $.proxy( this.synchronize, this ), component.name );
      
        mediator.subscribe( 'resize', $.proxy( this.chooseLayout, this ), 'viewportService' );
        mediator.subscribe( 'changeRange', $.proxy( this.reset, this ), 'viewportService' );
        mediator.subscribe( 'changeRange', $.proxy( this.renderPagination, this ), 'viewportService' );
    };

    // Initialize ui.component on mediator event 'ui.init'
    mediator.subscribe( 'ui.init', function() {
        $( '[data-' + component.slug + ']' ).each( function() {
            new WidgetBc( this );
        } );
    } );

    return WidgetBc;
} );