define([ 'jquery', 'underscore', 'component/core/mediator', 'viewportService', 'raven' ],
function( $, _, mediator, viewportService, Raven ) {

    'use strict';

    var
        queryParams = (function VetalioQueryParams() {
            var dataObj;
            if (dataObj = window.chip_currpage_json) {
                return {
                    containerID: dataObj.container_id || 'n/a',
                    titleText: encodeURI(dataObj.pagetitle) || 'n/a',
                    bartName: dataObj.bart_name || 'n/a'
                }
            }
            else if (dataObj = window.digitalData) {
                return {
                    containerID: dataObj.page.pageinfo.containerID || 'n/a',
                    titleText: encodeURI(dataObj.page.pageinfo.titleText) || 'n/a',
                    bartName: dataObj.page.category.bartName || 'n/a'
                }
            }
            else {
                console.error('Vetalio widget: neither digitalData nor chip_currpage_json could be found in window obj.');
            }
        }()),
        defaults = {
            scrollThreshold: viewportService.getViewport().viewportHeight * 1.5,
            url: '',
            redirectUrl: 'http://x.chip.de/linktrack/cloudsider/',
            queryParams: queryParams,
            templateId: '#tpl-widgetVetalio',
        },
        component = {
            name: 'WidgetVetalio',
            slug: 'widget-vetalio'
        };

    function WidgetVetalio( selector ) {
        this.$selector = $( selector );

        if ( !this.$selector.data( component.name ) ) {
            try {
                this.init();
            } catch( err ) {
                Raven.captureException( err, {
                    extra: {
                        module: component.name
                    }
                });
            }
        }
    }

    WidgetVetalio.prototype.init = function() {
        var args      = this.$selector.data( component.slug ),
            params    = _( args ).isObject() ? args : {};

        this.settings = _( params ).defaults( defaults );
        this.key      = this.settings.key || _( component.name + '_' ).uniqueId();
        this.template = _.template( $( this.settings.templateId ).html() );
        this.$widget  = this.$selector;

        this.loadData();
        this.subscribeToViewportService();
        this.$selector.data( component.name, this );
    };

    WidgetVetalio.prototype.subscribeToViewportService = function() {
        mediator.subscribe( 'change' , $.proxy( this.loadData, this ), 'viewportService' );
    };

    WidgetVetalio.prototype.loadData = function() {
        var viewport = viewportService.getViewport(),
            widgetOffsetTop = this.$widget.offset().top,
            loadWidget = viewport.scrollOffsetTop > ( widgetOffsetTop - this.settings.scrollThreshold );

        if ( !loadWidget || this.$widget.hasClass( 'is-loaded' ) ) {
            return;
        }

        this.$widget
            .addClass( 'is-loaded' );

        $.ajax({
            type: 'GET',
            url: this.settings.url,
            async: true,
            jsonpCallback: 'callback',
            contentType: 'application/json',
            dataType: 'jsonp',
            success: $.proxy( this.renderWidget, this )
        });
    };

    WidgetVetalio.prototype.renderRating = function( score ) {
        var grade = 'Schlecht',
            stars = 0,
            grades = {
                80: 'Sehr Gut',
                60: 'Gut',
                40: 'Befriedigend',
                20: 'Ausreichend',
                0: 'Schlecht'
        };

        _( grades ).each( function( schoolGrade, value ) {
            if ( score > parseInt( value ) ) {
                stars++;
                grade = schoolGrade;
            }
        });

        return {
            stars: stars,
            grade: grade,
            score: score
        };
    };

    WidgetVetalio.prototype.renderWidget = function( result ) {
        var items = result.items,
            link;

        _( items ).each( $.proxy( function( item, index ) {
            link = [ this.settings.redirectUrl,
                '%url%',
                '&container_id=', this.settings.queryParams.containerID,
                '&pagetitle=', this.settings.queryParams.titleText,
                '&provider=', item.name,
                '&=pagetype=', this.settings.queryParams.bartName
            ].join( '' );

            items[ index ].rating = this.renderRating( item.score_overall );
            items[ index ].providerLink = link.replace( '%url%', 'providerinfo/?url=' + item.affiliate_link );
            items[ index ].detailLink = link.replace( '%url%', 'detailinfo/?url=' + item.iframe_src );
        }, this ) );

        var html = this.template( {
            title: result.headline,
            subtitle: 'Partnerinhalt von <a href="' + result.url +'" target="_blank" rel="noindex, nofollow">trusted.de</a>',
            items: items
        });

        this.$widget
            .append( html );
    };

    mediator.subscribe( 'ui.init', function() {
      $( '[data-' + component.slug + ']' ).each( function() {
        new WidgetVetalio( this );
      } );
    } );

    return WidgetVetalio;
});
