define(['jquery','underscore', 'mediator', 'raven'], function ($, _, Mediator, Raven) {

    'use strict';

    var defaults = {
        'namespace': 'header',
        'mediator_channel': 'HEADER',
        'ajax_navigation': '',
        'template_submenu': '#tpl-headernavigation',
        'hover_delay': 250,
        'scrollDebounce': 50, // timeout for the scroll event
        'scrollSensibility': 5, // helper for touch devices
        'sticky': false
    };

    /**
     * The Header object
     *
     * @param selector
     * @param options
     * @returns {Header}
     * @constructor
     */
    function Header(selector) {
        this.$selector = $(selector);

        if ( !(this instanceof Header) ) {
            return new Header();
        }

        if ( !this.$selector.data('Header') ) {
            try {
                this.init();
            } catch (err) {
                Raven.captureException(err, {
                    extra: {
                        'module': 'Header'
                    }
                });
            }
        }
    };

    // ##### init()
    //
    // @param nav from annotationsLoader
    // @ return nav
    //
    Header.prototype.init = function () {
        var _this = this;
        this.has_touch = (("ontouchstart" in window) || ("msMaxTouchPoints" in navigator && navigator.msMaxTouchPoints > 0));

        // setting the parameter for this Element otherwise take the fallback parameter
        this.$selector.data('Header', this);
        this.params = this.$selector.data('header');
        this.params = _(this.params).isObject() ? this.params : {};
        this.settings = _(this.params).defaults(defaults);

        this.$window = $(window);
        this.$body = $('body');
        this.$navigation = $('.Navigation--Header', this.$selector);
        this.$search = $('.Search--Header', this.$selector);
        this.$overlay = $('.Header__Overlay', this.$selector);
        this.$searchtoggle = $('.Header__Button--ToggleSearch', this.$selector);
        this.$navigationtoggle = $('.Header__Button--ToggleNavigation', this.$selector);

        if ( this.settings.ajax_navigation ) {
            $.ajax({
                url: this.settings.ajax_navigation,
                dataType: 'jsonp',
                jsonp: false,
                jsonpCallback: 'navigation' || 'define',
                success: function (data) {
                    this.parseNavigation(data);

                    this.$subcategories = $('.Navigation__Link--HasSubmenu', this.$navigation);
                    //this.moveLinks();
                    this.bindListener();
                }.bind(this),
                error: function (data, err) {
                    Raven.captureException(err);
                }
            });
        } else {
            this.$subcategories = $('.Navigation__Link--HasSubmenu', this.$navigation);
            //this.moveLinks();
            this.bindListener();
        }

        if(this.settings.sticky === "true"){
            this.$selector.addClass('Header--Sticky'),
            this.$body.addClass('is-sticky')
                this.lastY = 0, // cached lastY on scroll
                this.currentY = 0, // cached current Y from window.scrollTop
                this.headerY = 0, // current header Y for dropdown fixes
                this.scrolling = false, // performance optimizer
                this.headerHidden = false, // header state
                this.blockHide = false, // block the hide functionality
                this.dropdownOpen = false // cache the state of dropdowns

            this.$window.on('scroll', $.proxy(this.requestScroll, this) );
        }
    };

    // ##### movelinks()
    //
    // searchs through the Markup and prepends every Link with Submenu this link as Sublink into the Submenu
    // and deactivates the Link of the ParentElement, so that on onClick only the submenue opens
    //
    Header.prototype.moveLinks = function () {
        _(this.$subcategories).each(function (element) {
            var $link = $(element).find('> a'),
                $backbutton = '<li class="Navigation__SubmenuHeadline"><a>Zurück</a></li>',
                $wrap = '<li></li>',
                $clone = $link.clone().wrap($wrap).parent();

            $(element).find('> ul').prepend($clone).prepend($backbutton);
        });
    };

    // ##### toggleSubmenu()
    //
    // the sibling ul of an link with Submenu will be set visible
    //
    Header.prototype.showSubmenu = function (e) {
        $(e.currentTarget).siblings('ul').addClass('is-visible');
        e.preventDefault();

    };

    // ##### bindListener()
    //
    //
    Header.prototype.bindListener = function () {
        if ( !this.has_touch ) {
            this.initMouseListener();
            this.initToggleListener();
        } else {
            this.initToggleListener();
            this.initTouchListener();
        }
    };

    // ##### initTouchListener()
    // d
    //
    Header.prototype.initTouchListener = function () {
        this.$subcategories.on('click.'+this.settings.namespace,'> a', this.showSubmenu);
        $('.Navigation__SubmenuHeadline').on('click.'+this.settings.namespace, $.proxy(this.checkOpenSubmenu, this, false ));
    };

    // ##### initMouseListener()
    // d
    //
    Header.prototype.initMouseListener = function () {
        var _this = this;

        this.$navigation
            .on(
                'mouseenter.'+this.settings.namespace+' '+'mouseleave.'+this.settings.namespace,
                'li',
                function(event){
                    _this.toggleListElements($(this),event.type);
                });
    };

    // ##### openOffCanvasListener()
    // d
    //
    Header.prototype.openOffCanvasListener = function () {
        var _this = this;

        this.$body
            .on('click.'+this.settings.namespace,
                'li',
                function(event){
                    _this.toggleListElements($(this),event.type);
                });
    };


    // ##### toggleListElements()
    //
    // reacts to the MouseEnter or MouseLeave on the Listelements in the Navigation
    // depending on the eventtype it sets or removes the active & Hover-Classes
    //
    Header.prototype.toggleListElements = function( $listelement, eventtype ) {
        var toggle_state = (eventtype == 'mouseenter' )? true:false;

        $listelement.toggleClass('is-active', toggle_state);
        _(_(function() {

            this.throwEvent(eventtype);
            $listelement.toggleClass('is-hover', toggle_state);
            this.clearHover();
        }).bind(this)).delay(this.settings.hover_delay);
    };

    // ##### initToggleListener()
    //
    // initializes the Toggle-Buttons in the header for showing
    // the searchbar or the navigation while in offCanvas
    //
    Header.prototype.initToggleListener = function() {

        this.$navigationtoggle.on('click.' + this.settings.namespace,
           // $.proxy(this.toggleElement, this)
           $.proxy(this.checkOpenSubmenu, this, true )
        );

        this.$searchtoggle.on('click.' + this.settings.namespace,
            $.proxy(this.checkOpenSearch, this )
        );

        this.$overlay.on('click.' + this.settings.namespace,
            $.proxy(this.overlayHandler, this )
        );
    };

    // ##### checkOpenSubmenu()
    //
    // Listens onclick to the ToggleTrigger Before it Toggle it's own Section active
    // it checks if another Section is open and closes that if so,
    // if multiple Submenues are open and no flag is set to close them all it only
    // closes one Submenu at the time
    //
    //
    Header.prototype.checkOpenSubmenu = function (close_all, event) {
        if(!close_all) { close_all = false; }
        var visible_level = $('.is-visible', this.$navigation),
            $navigation = this.$navigation,
            levels = 0,
            deepest_visible,
            delay_time = 0;

        if (this.$searchtoggle.hasClass('is-active')) {
            delay_time = 500;
            this.checkOpenSearch(event);
        }

        if ((close_all === false) && (visible_level.length > 0)){
            _(visible_level).each( function(visible_submenu){
                var levels_from_this = $(visible_submenu).parentsUntil($navigation).length;
                if(levels_from_this > levels) {
                    levels = levels_from_this;
                    deepest_visible = visible_submenu;
                }
            });
            $(deepest_visible).removeClass('is-visible');
        } else {
            visible_level.removeClass('is-visible');
            this.toggleElements(this.$navigationtoggle, this.$navigation, delay_time);
        }
    };

    // ##### checkOpenSearch()
    //
    // Listens onclick to the ToggleTrigger Before it Toggle it's own Section active
    // it checks if the other Section is open and closes that if so,
    //
    //
    Header.prototype.checkOpenSearch = function (event) {
        var delay_time = 0;

        if (this.$navigationtoggle.hasClass('is-active')) {
            delay_time = 500;
            this.checkOpenSubmenu(true, event);
        }
        this.toggleElements(this.$searchtoggle, this.$search, delay_time);
    };

    // ##### toggleElements()
    //
    // Function to set the state of the Togglebutton and the ToggleTarget
    //
    Header.prototype.toggleElements = function ($toggle_button, $target, delay) {
        var $overlay = this.$overlay,
            target = ($target === this.$search)?'searchbar':'offcanvas',
            status = ($toggle_button.hasClass('is-active'))?'close':'open';

        this.throwEvent(target+'.'+status);

        _(function() {
            $toggle_button.toggleClass('is-active');
            $overlay.toggleClass('is-active');

            $target.toggleClass('is-visible');
            $('body').toggleClass('is-locked');
        }).delay(delay);
    };

    // ##### throwEvent()
    //
    //
    //
    Header.prototype.throwEvent = function (message) {
        var evt,
            detail = { detail: { message: message} };
        if (typeof window.CustomEvent === 'function') {
            evt = new CustomEvent(this.settings.namespace, detail);
        } else {
            evt = document.createEvent('CustomEvent');
            evt.initCustomEvent(this.settings.namespace, false, false, detail);
        }
        window.dispatchEvent(evt);
        //console.log(this.settings.namespace+'//'+message+'//');
    };

    // ##### overlayHandler()
    //
    //
    //
    Header.prototype.overlayHandler = function (event) {

        if (this.$search.hasClass('is-visible')) {
            this.checkOpenSearch(event);
        } else if (this.$navigation.hasClass('is-visible')){
            this.checkOpenSubmenu(true, event);
        }
    };

    // ##### clearHover()
    //
    //
    //
    Header.prototype.clearHover = function () {
        this.$navigation
            .find('li.is-hover:not(".is-active")')
            .removeClass('is-hover');
    };

    // ##### parseNavigation()
    //
    // fetchs JSON with complete Navigation from external Source
    // parses the Markup and replaces the current Submenus according to the IDs
    Header.prototype.parseNavigation = function (data) {
        var $navigation = this.$navigation,
            template,
            template_data,
            submenu_data;

        // Underscore template declaration
        _.templateSettings.variable = "data";
        template = $('script'+this.settings.template_submenu).html();
        template_data = _.template(template);
        submenu_data = $('<div></div>').html(template_data(data));

        // looks for each available Submenu in the new NavigationMarkup
        // and replaces it with the old one
        _(data).each( function(dataEl, index){
            if (typeof (dataEl['group']) !== "undefined" ){
                var new_menu = $('#'+dataEl['group'], submenu_data),
                    old_menu = $('#'+dataEl['group'], $navigation);

                $(old_menu).replaceWith(new_menu);
            }
        });
    };

    // Sticky Header Functionen

    /**
     * behaviour when header dropdown is open
     */
    Header.prototype.onHeaderDropdownOpen = function () {

        if (!this.dropdownOpen) {
            this.scrollDebounce = 0;
            this.scrollSensibility = 0;
            this.dropdownOpen = true;
            this.currentY = this.$window.scrollTop();
            this.$selector.css('top', this.currentY);
            this.headerY = this.currentY;

            this.$selector.addClass('dropdown-open');
        }
    }

    /**
     * behaviour when header dropdown is closed
     */
    Header.prototype.onHeaderDropdownClosed = function () {

        this.dropdownOpen = false;
        this.$selector.removeClass('dropdown-open');

        // hide header if not visible
        if (this.currentY > this.headerY && this.headerY != 0) {
            this.toggleHeader(false);
            this.$selector.css('top', -50);
            this.headerY = -50;
        } else {
            this.$selector.css('top', 0);
            this.headerY = 0;
        }

        this.scrollSensibility = 7;
        this.scrollDebounce = 50;
    }



    /**
     * decouple scroll event to save performance
     */
    Header.prototype.requestScroll = function (event) {
        if (!this.scrolling) {
            this.scrolling = true;
            _.debounce(function () {
                this.doScroll();
            }.bind(this), this.settings.scrollDebounce)()
        }
    }

    /**
     * scroll logic
     */
    Header.prototype.doScroll = function() {
        // save the current Y
        this.currentY = this.$window.scrollTop();

        if (Math.abs(this.currentY - this.lastY) > this.settings.scrollSensibility) {
            if (this.lastY > this.currentY) {
                // scroll up
                this.toggleHeader(false);
            } else {
                // scroll down
                this.toggleHeader(true);
            }
        }

        // scrolling is done, ready for next tick
        this.scrolling = false;
        this.lastY = this.currentY;
    }

    /**
     * hide header if header is not hidden and the hiding is allowed
     */
    Header.prototype.toggleHeader = function (hide) {
        var status = null;
        if (this.headerHidden === hide) {
            return true;
        } else if ( this.blockHide || this.dropdownOpen ) {
            return true;
        }

        this.$selector.toggleClass('is-hidden',hide);

        if(!hide && this.headerY < 0) {
            this.$selector.css('top', 0);
            this.headerY = 0;
        }

        this.headerHidden = hide;
        status = (this.headerHidden !== true)?'reveal':'hide';

        this.throwEvent('sticky.'+status);
    }




    Mediator.subscribe('ui.init', function(){
        $('[data-header]').each(function() {
            new Header(this);
        });
    });

    return Header;

})
