define( [ 'jquery', 'underscore', 'viewportService', 'component/core/mediator', 'raven', 'LazyImage', 'hammerjs', 'hammerjs-jquery' ],
    function( $, _, viewportService, mediator, Raven, LazyImage, Hammer ) {

      'use strict';

      var defaults = {
        sync: [],
        asNavFor: '',
        slides: 1,          // string or object {small: 1, medium:3, large: 5}
        paginate: 'slide',  // slide or page
        pagination: false
      }, component = {
        name: 'Carousel',
        slug: 'carousel',
        channel: 'ui.carousel'
      };

      function Carousel( selector ) {
        this.$selector = $( selector );

        if ( !( this instanceof Carousel ) ) {
          return new Carousel();
        }

        if ( !this.$selector.data( component.name ) ) {
          try {
            this.init();
          } catch( err ) {
            Raven.captureException( err, {
              extra: {
                module: component.name
              }
            });
          }
        }
      }

      // #### init()
      //
      // Makes sure the element won’t get initialized twice by saving the component
      // into element.data( 'LazyImage' ).
      // Triggers `chooseLayout()` to setup the right layout depending on the current viewport.
      // Binds *touchstart*, *click* and *swipe* to the list, the prevNext buttons and the navigation
      // buttons. In the end it subscribes to **mediator* to sync with other **carousel** and with
      // the **viewportService** to trigger `chooseLayout()` again.
      //
      Carousel.prototype.init = function() {
        var args = this.$selector.data( component.slug ),
            params = _( args ).isObject() ? args : {};

        this.settings   = _( params ).defaults( defaults );
        this.key        = this.settings.key || _( component.slug + '_' ).uniqueId();


        this.position = 0;
        this.slide = 0;
        this.page = 0;
        this.isNavigation = !_( this.settings.asNavFor ).isEmpty();

        this.$carousel = this.$selector;
        this.$list = this.$carousel.find( 'ul' );
        this.$slides = this.$carousel.find( '.Carousel__Slide' );
        this.$prevNext = this.$carousel.find( '.Carousel__Button--Next, .Carousel__Button--Prev' );
        this.$next = this.$carousel.find( '.Carousel__Button--Next' );
        this.$prev = this.$carousel.find( '.Carousel__Button--Prev' );
        this.$pagination = this.$carousel.find( '.Carousel__Pagination' );
        this.$counter = this.$carousel.find( '.Carousel__Counter' );
        this.$stage = this.$carousel.find( '.Carousel__Stage' );

        this.chooseLayout();
        this.bindEvents();
        this.subscribeToMediator();

        this.$selector.data( component.name, this );
      };


      // #### chooseLayout()
      //
      // Chooses the right layout. Checks in which size *(small, medium, large)* the page currently is
      // displayed by including the **viewportService** and then fetches the corresponding
      // part of `this.settings[ view ]`. After calculating the widths of slides and list it
      // generates the total pages and then triggers `renderPagination()`
      // to display counter and pagination and finally triggers `goTo()` to jump to the very
      // first slide.
      //
      Carousel.prototype.chooseLayout = function() {
        var view = viewportService.getViewport().viewportRange,
            pagingType = ( _.isObject( this.settings.paginate ) && this.settings.paginate[ view ] ) ? this.settings.paginate[ view ] : this.settings.paginate,
            slidesPerPage = ( _.isObject( this.settings.slides ) && this.settings.slides[ view ] ) ? this.settings.slides[ view ] : this.settings.slides;

        this.view = view;
        this.width = this.$carousel.width();
        this.slide = ( this.slide > this.slides ) ? this.slides : this.slide;
        this.slides = this.$slides.length;
        this.pages = Math.ceil( this.slides / slidesPerPage );
        this.skip = pagingType === 'page' ? slidesPerPage : 1;
        this.slidesPerPage = slidesPerPage;
        this.slidesWidth = this.width / slidesPerPage;
        this.listWidth = ( this.pages + 1 ) * this.width;

        this.$carousel
            .toggleClass( 'Carousel--Nav', this.isNavigation )
            .addClass( 'is-loaded' );

        this.$list
            .width( this.listWidth );

        this.$slides
            .width( this.slidesWidth )
            .removeClass( 'is-hidden' )
            .find('.Carousel__ImageCaption')
            .width( this.slidesWidth );

        this.renderPagination();
        this.goTo( this.slide );
      };

      // #### renderPagination()
      //
      // Renders the navigation buttons, to jump to a specific page if
      // enabled. If the carousel is used as navigation for another carousel,
      // showing the pagination is disabled. Since the amount of pages can vary
      // in the different viewportRanges (i.e. 2 slides on small view and 4 slides
      // on medium and large view), the pagination has to be rendered again
      // if the amount of `this.pages` change.
      //
      Carousel.prototype.renderPagination = function() {
        var buttons = _( this.pages ).range(),
            html = [];

        if ( _.isEmpty( this.$pagination ) || this.isNavigation ) {
          return;
        }

        if ( this.$pagination.find('.Carousel__Nav').length !== this.pages ) {
          _( buttons ).each( function() {
            html.push('<button type="button" class="Carousel__Nav">&nbsp;</button>');
          } );

          this.$pagination
              .removeClass( 'is-hidden' )
              .html( html.join( '' ) )
              .find( '.Carousel__Nav' )
              .eq( this.page )
              .addClass( 'is-active' );
        }
      };

      // #### goTo()
      //
      // Takes care of loading the correct slide and calculating the position
      // of the list, which gets translated via css3 to the left and right.
      // If used as navigation, the currently active slide gets a css helper
      // class to make it visible. The styling itself is not part of this
      // component and has to be done seperately. If syncing the carousel
      // with another is enabled it invokes publishing a **mediator** event of the keys to sync
      // and the current slide which will trigger a `goTo()` for carousels to be synced.
      //
      Carousel.prototype.goTo = function ( slide ) {
        var lastSlide = this.isNavigation ? this.slides - 1 : this.slides - this.slidesPerPage,
            threshold = Math.floor( this.slidesPerPage / 2 ),
            publishMediatorEvent = function( slide ) {
              mediator.publish( this, slide, component.channel );
            };

        if ( slide >= lastSlide ) {
          this.slide = lastSlide;
        } else if ( slide <= 0 ) {
          this.slide = 0;
        } else {
          this.slide = slide;
        }

        this.lazyLoad( this.slide );

        this.$prev
            .toggleClass( 'is-disabled', this.slide < 1 );

        this.$next
            .toggleClass( 'is-disabled', this.slide >= lastSlide );

        if ( this.isNavigation ) {
          if ( this.slide > threshold && this.slide < this.slides - threshold ) {
            this.position = this.slidesWidth * ( this.slide - threshold );
          } else if (this.slide <= threshold ) {
            this.position = 0;
          }

          this.$slides
              .eq( this.slide )
              .addClass( 'is-active' )
              .siblings()
              .removeClass( 'is-active' );

          this.$list
              .css( 'transform', 'translate(' + (this.position * -1) + 'px, 0px)' );

        } else {
          this.position = this.slidesWidth * this.slide * -1;
          this.page = Math.ceil( this.slide / this.slidesPerPage );

          this.$list
              .css( 'transform', 'translate(' + this.position + 'px, 0px)' );

          this.$pagination
              .find( '.Carousel__Nav' )
              .removeClass( 'is-active' )
              .eq( this.page )
              .addClass( 'is-active' );

          if ( !_.isEmpty( this.$counter ) ) {
            this.$counter
                .find( 'span' )
                .text( this.slide + 1 );
          }
        }

        if ( !_( this.settings.sync ).isEmpty() ) {
          _( this.settings.sync ).invoke( publishMediatorEvent, this.slide );
        }
      };

      // #### bindEvents()
      //
      // Binds a namespaced version of `touchstart` and `click` events
      // to the next and previous buttons and the navigation links *(readded if the viewportRange changes)*.
      // If the carousel serves as navigation (i.e. thumbnails) for a another carousel, the
      // slides itself get a click and touch handler applied to activate the current slide. If
      // it does not serve as navigation, the list gets `pan` events bound via **hammerjs**.
      // to enable swiping through the elements.
      //
      Carousel.prototype.bindEvents = function() {
        this.$prevNext
            .on( 'touchstart.' + this.key, $.proxy( this.skipToSlide, this ) )
            .on( 'click.' + this.key, $.proxy( this.skipToSlide, this ) );

        this.$pagination
            .on( 'touchstart.' + this.key, '.Carousel__Nav', $.proxy( this.skipToPage, this ) )
            .on( 'click.' + this.key, '.Carousel__Nav', $.proxy( this.skipToPage, this ) );

        if ( this.isNavigation ) {
          this.$slides
              .on( 'touchstart.' + this.key, $.proxy( this.activateSlide, this ) )
              .on( 'click.' + this.key, $.proxy( this.activateSlide, this ) );
        } else {
          this.$carousel.hammer()
              .on( 'panright panleft', $.proxy( this.stickToFinger, this ) )
              .on( 'panend', $.proxy( this.fixPosition, this ) );
        }
      };

      // #### skipToSlide()
      //
      Carousel.prototype.skipToSlide = function( event ) {
          console.log('skipToSlide');
        var $button = $( event.target ).closest( '.Carousel__Button' );

        if ( $button.hasClass( 'is-disabled') ) {
          return;
        } else if ( $button.hasClass( 'Carousel__Button--Next' ) ) {
          this.goTo( this.slide + this.skip );
        } else if ( $button.hasClass( 'Carousel__Button--Prev' ) ) {
          this.goTo( this.slide - this.skip );
        }

        event.stopPropagation();
        event.preventDefault();
      };

      Carousel.prototype.skipToPage = function( event ) {
        var $button = $( event.target ).closest( '.Carousel__Nav' ),
            slide = $button.prevAll().length * this.slidesPerPage;

        $button
            .addClass( 'is-active' )
            .siblings()
            .removeClass( 'is-active' );

        this.goTo( slide );

        event.stopPropagation();
        event.preventDefault();
      };

      Carousel.prototype.activateSlide = function( event ) {
        var $slide = $( event.target ).closest( '.Carousel__Slide' ),
            slide = $slide.prevAll().length;

        this.goTo( slide );

        event.stopPropagation();
        event.preventDefault();
      };

      // #### lazyLoad()
      //
      // Loops over all slides up to `slide + this.slidesPerPage` and fetches all
      // images inside of it, which should be lazy loaded with **LazyImage**. For each
      // element, the `load()` function of **LazyImage** is triggered. Once
      // triggered the image will also replace it source on `window.resize()`, if different
      // sources are provided for each viewportRange. See **LazyImage** for more information.
      //
      Carousel.prototype.lazyLoad = function( slide ) {
        var $images,
            loadImage = function() {
              var lazyImage = $(this).data('LazyImage');

              if ( lazyImage ) {
                lazyImage.load();
              } else {
                lazyImage = new LazyImage( this );
                lazyImage.load();
              }
            };

        $images = this.$slides
            .filter( ':lt(' + slide + this.slidesPerPage + ')' )
            .find( '.LazyImage' );

        _( $images ).invoke( loadImage );
      };

      Carousel.prototype.fixPosition = function( event ) {
        var position = this.$list.data( 'position' ),
            move = event.gesture;

        if (move.pointerType === 'mouse') {
          return;
        }

        if ( event.type === 'panend') {

          if ( Math.abs( event.gesture.deltaX ) > this.slidesWidth / 2 ) {
            if ( event.gesture.deltaX < 0 ) {
              this.goTo( this.slide + 1 );
            } else {
              this.goTo( this.slide - 1 );
            }
          } else {
            this.goTo( this.slide );
          }
        }
      };

      Carousel.prototype.reset = function( event ) {
        this.page = 0;
        this.slide = 0;
        this.position = 0;

        this.goTo( this.slide );
      };

      Carousel.prototype.stickToFinger = function( event ) {

        if ( event.gesture.pointerType === 'mouse' ) {
          return;
        }

        this.$list
            .css( 'transform', 'translate3d(' + (  this.position + event.gesture.deltaX * 1.5 ) +'px, 0px, 0px)');
      };

      Carousel.prototype.synchronize = function( slide ) {
        if ( this.slide !== slide ) {
          this.goTo( slide );
        }
      };

      // #### subscribeToViewportService()
      //
      // Subscribes to changes of the viewport and triggers 'loadImage()'
      Carousel.prototype.subscribeToMediator = function() {
        mediator.subscribe( this.key, $.proxy( this.synchronize, this ), component.name );

        mediator.subscribe( 'resize', $.proxy( this.chooseLayout, this ), 'viewportService' );
        mediator.subscribe( 'changeRange', $.proxy( this.reset, this ), 'viewportService' );
        mediator.subscribe( 'changeRange', $.proxy( this.renderPagination, this ), 'viewportService' );
      };

      // Initialize ui.component on mediator event 'ui.init'
      mediator.subscribe( 'ui.init', function() {
        $( '[data-' + component.slug + ']' ).each( function() {
          new Carousel( this );
        } );
      } );

      return Carousel;
    } );