// The component handles loading of images either lazy, or replacing
// a existing one with a different one if the viewportRange changes. 
// To ensure the layout stays the same while a placeholder is being shown, an embedded
// svg is used. By altering the viewBox its possible to give the placeholder the same
// aspect ratio as the original image.  
// Since the component is only replacing src attribute of the image, any other attribute
// needed for any usecase, like an alt-attribute, needs to be added to the placeholder image
//
// ##### Options *(object)*:
// - **key:** "myKey"  
// *Used for namespacing the events*
// - **src:** {small: "1.jpg", medium: "2.jpg", large: "3.jpg"} | "lazy.jpg"  
// *Source of the image provided for each size or one size for all* 
// - **scrollThreshold:** 400  
// *Scrolling distance when the image gets loaded*
// - **load:** "auto" | "manual"  
// *Manual loading by another module, or default auto load*
//
// ##### Example 1: 
// ###### Manual loading of one image
// ```
// $options = array("src" => "lazy.jpg", "load" => "manual");
// <div class="LazyImage" 
//   data-lazy-image='<?= json_encode($options); ?>'
//   data-lazy-image-text="Image is being loaded!">
//     <img alt="ma bacon!" src="data:image/svg+xml;charset=utf-8,%3Csvg xmlns%3D'http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg' viewBox%3D'0 0 4 3'%2F%3E">
// </div>
// ```
// ##### Example 2:
// ###### Auto loading of a different image for each size, with the mobile size as placeholder
// ```
// $options = array("src" => array(
//  "small" => "20x20.jpg", 
//  "medium" => "40x40.jpg",
//  "large" => "60x60.jpg")
// );
// <div class="LazyImage" 
//   data-lazy-image='<?= json_encode($options); ?>'
//   data-lazy-image-text="Bild lädt...">
//     <img alt="ma bacon!" src="20x20.jpg">
// </div>
// ```
//
define( [ 'jquery', 'underscore', 'viewportService', 'component/core/mediator', 'raven' ],
    function( $, _, viewportService, mediator, Raven ) {

    'use strict';

    var defaults = {
        scrollThreshold: viewportService.getViewport().viewportHeight / 2,
        loadFor: ['small', 'medium', 'large'],
        load: 'auto',
        placeholder: 'data:image/svg+xml;charset=utf-8,%3Csvg xmlns%3D\'http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg\' viewBox%3D\'0 0 4 3\'%2F%3E'
    }, component = {
        name: 'LazyImage',
        slug: 'lazy-image',
        channel: 'ui.lazy-image'
    };

    function LazyImage( selector ) {
        this.$selector = $( selector );

        if ( !( this instanceof LazyImage ) ) {
          return new LazyImage();
        }

        if ( !this.$selector.data( component.name ) ) {
            try {
                this.init();
            } catch( err ) {
                Raven.captureException( err, {
                    extra: {
                        module: component.name
                    }
                });
            }
        }
    }

    // #### init()
    //
    // Makes sure the element won't get initialized twice by saving the component into 
    // `element.data( 'LazyImage' )`.  
    // Triggers `loadImage()` for any image currently visible in the viewport and
    // subscribes to the **viewportService** to trigger `loadImage()` on scrolling and
    // resizing.
    //
    LazyImage.prototype.init = function() {
        var args = this.$selector.data( component.slug ),
            params = _( args ).isObject() ? args : {};

        this.settings   = _( params ).defaults( defaults );
        this.key        = this.settings.key || _( component.slug + '_' ).uniqueId();


        this.source = null;
        this.load = function() {
            this.settings.load = 'auto';
            this.loadImage();
        };

        this.$lazyImage = this.$selector;

        this.loadImage();
        this.subscribeToViewportService(); 

        this.$selector.data( component.name, this );
    };

    // #### loadImage()
    //
    // Replaces the placeholder with the original image.  
    // As soon as its replaced a helper class gets added to the element
    // to hide the loading message. Images with load type *manual* are skipped
    // until the load settings are changed to auto which can be done like that:  
    // `$myImage.data( 'LazyImage' ).load();`.
    //
    LazyImage.prototype.loadImage = function() {
        var viewport = viewportService.getViewport(),
            loadImage = viewportService.inY( this.$lazyImage, this.settings.scrollThreshold ),
            newSrc = null,
            source = this.settings.src;

        if ( this.source !== source ) {
            newSrc = source;
        }

        if ( this.settings.load === 'manual' || !loadImage || !newSrc || this.isFailed ) {
            return;
        }

        this.$lazyImage
            .addClass( 'is-loaded' )
            .find( 'img' )
                .on( 'error', $.proxy( this.loadError, this ) )
                .attr( 'src', newSrc );

        this.source = newSrc;
    };

    LazyImage.prototype.loadError = function() {
        this.isFailed = true;

        this.$lazyImage
            .removeClass( 'is-loaded' )
            .addClass( 'is-failed' )
            .attr( 'data-lazy-image-text', 'Bild nicht verfügbar...')
            .find( 'img' )
                .attr( 'src', this.settings.placeholder )
                .attr( 'alt', '');
    };

    // #### subscribeToViewportService()
    //
    // Subscribes to **viewportService** and listens to any changes of 
    // the current viewport and triggers `loadImage()`.
    //
    LazyImage.prototype.subscribeToViewportService = function() {
        mediator.subscribe( 'load', $.proxy( this.loadImage, this ), component.channel );
        mediator.subscribe( 'scroll', $.proxy( this.loadImage, this ), 'viewportService' );
    };

    // Initialization happens when **mediator** publishes the event `ui.init`.
    mediator.subscribe( 'ui.init', function() {
        $( '[data-' + component.slug + ']' ).each( function() {
            new LazyImage( this );
        } );
    } );

    return LazyImage;
} );